package src.core;


import java.util.ArrayList;
import java.util.Vector;

import src.distort.Normalizer;
import src.distort.Pair;
import src.distort.Shaker;
import src.mlp.PatternSymbol;
import src.util.ImageCreator;


public class MainController {
	private Vector <Sample> sampleList;	
	private int TOTAL_CLASSES = 10; /*Nmero de classes a ser utilizado neste teste. O nmero mximo pode ser 77*/
	private int TOTAL_FOLDS = 10; /*Nmero de "passagens" da cross validation*/
	private int	TOTAL_SYMBOLS = 154; /*Nmero de simbolos escrito por cada usurio*/
	private int TOTAL_DISTORTIONS = 2; /*Adicionar distores melhora a taxa de reconhecimento em at 10%, mas o tempo de processamento aumenta muito.*/
	
	
	public static void main(String[] args) {
		new MainController().performCrossValidation();
	}
	
	/*Classe que executa o teste de acordo com os parmetros definidos nas contantes acima*/
	public void performCrossValidation(){		
		MainController mainController = new MainController();		
		sampleList = new InputFileReader().readFile();		
		Shaker t = new Shaker();
		Normalizer n= new Normalizer();
		double recognitionRateSum =0.0;
		
		for (int fold = 0 ; fold < TOTAL_FOLDS ; fold++){			
			System.out.println("fold: "+fold);
			ArrayList <PatternSymbol> trainingSymbols  = new ArrayList <PatternSymbol> () ;
			ArrayList <PatternSymbol> recognizingSymbols = new ArrayList <PatternSymbol> () ;
			ClassifierController  classifier = new ClassifierController();			

			for(int writerNumber =0 ; writerNumber<10 ; writerNumber++){
				for(int symbolNumber =0 ; symbolNumber < TOTAL_SYMBOLS ; symbolNumber++){					
					int symbolClass;				
					if (symbolNumber<77)
						symbolClass =symbolNumber;
					else
						symbolClass = symbolNumber-77;
					if(symbolClass < TOTAL_CLASSES ){
						PointList character = sampleList.get(writerNumber).symbols.get(symbolNumber);						
						character = t.interpolate(character);						
						character = n.fillTheWholeMatrix(character);
						int [][]expandida = mainController.expandPoints(2, character);						
						int [][]mini = mainController.miniaturize(expandida);						
						PatternSymbol symbol = pointListToPS(character,mini,symbolClass);
						/*Pode-se gerar uma imagem do smbolo descomentando as linhas abaixo.*/
//						String nome = Integer.toString(symbolNumber);
//						nome = nome.concat("writer");
//						nome = nome.concat(Integer.toString(writerNumber));
//						nome = nome.concat(".pbm");
//						ImageCreator ic = new ImageCreator();
//						ic.createImage(nome, expandida);
						
						if((writerNumber == fold && symbolNumber<77) || ( writerNumber == (fold+5)%10 && symbolNumber>=77)){
							recognizingSymbols.add(symbol);
						}						
						else{
							trainingSymbols.add(symbol);							
							
							for(int distortNumber =0 ; distortNumber < TOTAL_DISTORTIONS ; distortNumber++){
								//System.out.println("distoro numero: "+distortNumber);
								PointList tremido = new PointList(character);
								tremido = t.shake(tremido);
								int[][]matrizTremida = expandPoints(2, tremido);								
								mini = miniaturize(matrizTremida);
								PatternSymbol shakedSymbol = pointListToPS(tremido, mini, symbolClass);
								trainingSymbols.add(shakedSymbol);
							}
						}
					}
				}
			}						
			recognitionRateSum += classifier.run(trainingSymbols,recognizingSymbols,TOTAL_CLASSES);			
		}
		System.out.println("Taxa de reconhecimento das "+TOTAL_FOLDS+" passagens: "+recognitionRateSum/TOTAL_FOLDS+"%.");
	}
	
	
	private boolean canExpand(int fator,int [][]nova, int i,int j){		
		if(i>=fator)
			if(i+fator<nova.length)
				if(j>=fator)
					if(j+fator<nova[0].length){						
						return true;
					}		
		return false;
	}
	
	private int [][] expandPoints(int fator,PointList original){
		Matrix nm = new Matrix();
		int [][] nova = nm.theMatrix;
		for (int i=0;i<original.getList().size();i++){
			Pair atual = original.getList().get(i);
				if(canExpand(fator,nova,atual.getY(),atual.getX()))
					for(int m=atual.getY()-fator ; m<=atual.getY()+fator ; m++)
						for(int n=atual.getX()-fator ; n<=atual.getX()+fator ; n++){
							nova[m][n]=1;
					}
		}
		return nova;
	}
	
	private int [][] miniaturize (int [][] original){
		int [][]mini = new int[20][20];
		for(int y =0;y<original.length;y++){
			for(int x =0; x<original[0].length; x++)
				if(original[y][x]==1)
					mini[y/25][x/25] = 1;
		}
		return mini;
	}
	
	
//	private void printMatrix (int [][] original){		
//		for(int y =0;y<original.length;y++){
//			for(int x =0; x<original[0].length; x++)
//				System.out.print(original[y][x]);
//			System.out.println("");
//		}
//	}

	private PatternSymbol pointListToPS(PointList caractere, int [][] miniature,int classe) {
		PatternSymbol ps = new PatternSymbol();
		ps.setNumStrokes(caractere.getNumTracos());
		ps.setIni(caractere.getFirstPointPosition());
		ps.setEnd(caractere.getLastPointPosition());
		ps.setIniEndDirection(caractere.getFirstToLastPointDirection());
		ps.setDensity(caractere.getVerticalDensity());
		ps.setShape(caractere.getShape());
		ps.setSymbol(matrizDeInteirosParaVetorDouble(miniature));
		ps.setPattern(classe);
		return ps;
	}
	public void imprimeVetor(double []v){
		int tamanho = v.length;
		for (int i =0;i<tamanho;i++)
			System.out.printf("%5.3f ",v[i]);	
		System.out.println("");
	}

	private double[] matrizDeInteirosParaVetorDouble(int[][] miniature) {
		double[] array = new double[miniature.length*miniature[0].length];
		for(int i=0;i<miniature.length;i++)
			for(int j=0;j<miniature[0].length;j++)
				array[miniature[0].length*i+j]=miniature[i][j];
		return array;
	}

}
