package src.core;

import java.util.Vector;

import src.distort.Pair;

/* Cada lista de pontos contm todas as informacoes sobre um simbolo escrito por um determinado usurio.*/
public class PointList {
	private int xMin =2000;
	private int xMax =-1;
	private int yMin =2000; 
	private int yMax =-1;
	private int firstPointPosition;
	private int lastPointPosition;
	private int firstToLastPointDirection;
	private int verticalDensity;
	private int shape;
	private int beginX =0;	
	private int endX =0;
	private int beginY =0;
	private int endY =0;
	private int numTracos;	
	private Vector <Pair> list;	

	/*Cria uma nova pointList com os mesmos dados da antiga*/
	public PointList(PointList oldPointList){
		this.xMin = oldPointList.xMin;
		this.xMax = oldPointList.xMax;
		this.yMin = oldPointList.yMin;
		this.yMax = oldPointList.yMax;	
		this.numTracos = oldPointList.numTracos;
		this.verticalDensity = oldPointList.verticalDensity;
		this.firstToLastPointDirection = oldPointList.firstToLastPointDirection;
		this.shape = oldPointList.shape;
		this.lastPointPosition = oldPointList.lastPointPosition;
		this.firstPointPosition = oldPointList.firstPointPosition;
		list = new Vector <Pair>();
		for(int i =0;i<oldPointList.list.size();i++){
			
			Pair atual = oldPointList.list.get(i);
			Pair novo = new Pair(atual.getX(),atual.getY());
			list.add(novo);
		}
	}
	
	public PointList(){
		this.xMin =2000;
		this.xMax =-1;
		this.yMin =2000; 
		this.yMax =-1;		
		list = new Vector <Pair>();
	}
	
	
	public void updateFeatures() {		
		firstPointPosition = analyzePosition(beginX, beginY);		
		lastPointPosition = analyzePosition(endX, endY);
		firstToLastPointDirection = analyzeDirection();
		shape = analyzeShape();
		verticalDensity = analizeVerticalDensity();		
	}
	
	private int analyzeShape() {
		int height = Math.abs(xMax - xMin);
		int width = Math.abs(yMax - yMin);		
		double widthToHeightRatio = width/height;
		if(widthToHeightRatio > 1.2)
			return 0;
		if (widthToHeightRatio > 0.8)
			return 1;
		return 2;
	}
	
	private int analyzeDirection() {
		final double SEVENTY_DEGREE_TANGENT = 2.7474774194546222787616640264977; /*tan(70)*/
		final double TWENTY_DEGREE_TANGENT = 0.36397023426620236135104788277683; /*tan(20)*/		
		int verticalOffset = beginY - endY;
		int horizontalOffset = endX - beginX;				
		double tangent;
		if(horizontalOffset != 0)
			tangent = (double)verticalOffset/horizontalOffset;
		else 
			tangent = 1.0E100;/*Proximo a pi/2 e -pi/2 o valor da tangente tende ao infinito*/		
		if(horizontalOffset >0){
			if(tangent < -SEVENTY_DEGREE_TANGENT)
				return 4;
			if(tangent < -TWENTY_DEGREE_TANGENT)
				return 3;
			if(tangent < TWENTY_DEGREE_TANGENT)
				return 2;
			if(tangent < SEVENTY_DEGREE_TANGENT)
				return 1;
			else
				return 0;				
		}else if(horizontalOffset <0){
			if(tangent < -SEVENTY_DEGREE_TANGENT)
				return 0;
			if(tangent < -TWENTY_DEGREE_TANGENT)
				return 7;
			if(tangent < TWENTY_DEGREE_TANGENT)
				return 6;
			if(tangent < SEVENTY_DEGREE_TANGENT)
				return 5;
			else
				return 4;			
		}
		else{
			if(verticalOffset <0)
				return 4;
			else
				return 0;
		}
	}
	
	private int analyzePosition(int x, int y) {		
		int mediumX = (xMax + xMin)/2;
		int mediumY = (yMax + yMin)/2;
		if(x<mediumX){
			if(y<mediumY)				
				return 3;			
			else				
				return 2;			
		}else{
			if(y<mediumY)
				return 0;
			else
				return 1;
		}
	}
	private int analizeVerticalDensity(){
		int height = Math.abs(yMax - yMin);		
		int weightedTotal=0;
		int simpleTotal =0;
		int [] lines = new int[500];		
		for(int i =0; i<list.size();i++){
			int value = list.get(i).getY();
			if(value >= 0 && value <500)
				lines[list.get(i).getY()]++;			
		}		
		for(int i=0;i<lines.length;i++){
			simpleTotal += lines[i];
			weightedTotal += lines[i]*i;
		}		
		int peak = weightedTotal /simpleTotal;		
		if(peak < (yMin+(height/5)*2))
			return 2;
		if(peak < (yMin+(height/5)*3))
			return 1;
		else
			return 0;		
	}
	
	
	public void setXIni(int ini) {		
		beginX = ini;
	}
	public void setXFim(int fim) {
		endX = fim;
	}
	public void setYIni(int ini) {
		beginY = ini;
	}
	public void setYFim(int fim) {
		endY = fim;
	}
	public int getNumTracos() {
		return numTracos;
	}

	public void setNumTracos(int numTracos) {
		this.numTracos = numTracos;
	}

	public Vector<Pair> getList() {
		return list;
	}

	public void setList(Vector<Pair> list) {
		this.list = list;
	}
	
	public void increaseStrokesNumber(){
		numTracos ++;
	}

	public int getXMin() {
		return xMin;
	}

	public void setXMin(int min) {
		xMin = min;
	}

	public int getXMax() {
		return xMax;
	}

	public void setXMax(int max) {
		xMax = max;
	}

	public int getYMin() {
		return yMin;
	}

	public void setYMin(int min) {
		yMin = min;
	}

	public int getYMax() {
		return yMax;
	}

	public void setYMax(int max) {
		yMax = max;
	}

	public int getFirstPointPosition() {
		return firstPointPosition;
	}

	public void setFirstPointPosition(int firstPointPosition) {
		this.firstPointPosition = firstPointPosition;
	}

	public int getLastPointPosition() {
		return lastPointPosition;
	}

	public void setLastPointPosition(int lastPointPosition) {
		this.lastPointPosition = lastPointPosition;
	}

	public int getFirstToLastPointDirection() {
		return firstToLastPointDirection;
	}

	public void setFirstToLastPointDirection(int firstToLastPointDirection) {
		this.firstToLastPointDirection = firstToLastPointDirection;
	}

	public int getVerticalDensity() {
		return verticalDensity;
	}

	public void setVerticalDensity(int verticalDensity) {
		this.verticalDensity = verticalDensity;
	}

	public int getShape() {
		return shape;
	}

	public void setShape(int shape) {
		this.shape = shape;
	}
}


