package src.treeController;

import java.util.ArrayList;

import src.hashFunctions.IHashFunction;
import src.mlp.NetworkConfigs;
import src.mlp.PatternSymbol;
import src.mlp.statistics.Statistics;

/* Cada n interno da rvore de reconhecimento funciona apenas como um filtro. Ele analiza uma determinada caracterstica do smbolo e 
 * indica qual dos seus ramos  indicado para tratar este smbolo.*/
public class InternalNode {
	private int numberOfClasses; 
	private IHashFunction hashFunction;
	private boolean hasLeaves; /*Variavel booleana indica se abaixo deste nivel esto as folhas.*/
	private ArrayList <InternalNode> nodes = new ArrayList <InternalNode> ();
	private ArrayList <Leaf> leaves = new ArrayList <Leaf> ();
	
	
	public InternalNode(ArrayList <IHashFunction> hashFunctions, int thisNodeLevel, NetworkConfigs netSize){		
		int totalLevels = hashFunctions.size();		
		this.hashFunction = hashFunctions.get(thisNodeLevel);		
		this.numberOfClasses = hashFunction.getNumberOfClasses();		
		if(thisNodeLevel == totalLevels-1){ /*Abaixo deste nvel s existem folhas*/			
			this.hasLeaves = true;		
			for(int i =0; i<numberOfClasses;i++){
				Leaf newLeaf = new Leaf(netSize);
				leaves.add(newLeaf);
			}
		}else{			
			this.hasLeaves = false;		
			for(int i =0; i<numberOfClasses;i++){
				InternalNode newNode = new InternalNode(hashFunctions,thisNodeLevel+1,netSize);
				nodes.add(newNode);
			}
		}
	}

	public int recognize(PatternSymbol ps){
		return getLeaf(ps).recognize(ps);
	}	
	
	/*Devolve a folha indicada para reconhecer o smbolo*/
	public Leaf getLeaf (PatternSymbol ps){
		int hashKey = this.hashFunction.apply(ps);
		if(this.hasLeaves){
			Leaf appropriateLeaf = leaves.get(hashKey);
			return appropriateLeaf;
		}else{
			InternalNode nextLevelNode = nodes.get(hashKey);
			return nextLevelNode.getLeaf(ps);
		}		
	}
	
	/*Calcula a taxa de reconhecimento da sub-arvore iniciada neste n.*/
	public Statistics getStatistics(){
		int correctSum = 0;
		int totalSum = 0;
		int numberOfClasses = hashFunction.getNumberOfClasses();
		if(this.hasLeaves){
			for(int i =0; i<numberOfClasses;i++){				
				Leaf leaf = leaves.get(i);
				Statistics temp = leaf.getNet().getConfusionMatrix().getStatistics();
				correctSum += temp.getCorrectPatterns();
				totalSum += temp.getTotalPatterns();
			}
		}else{
			for(int i =0; i<numberOfClasses;i++){				
				InternalNode node = nodes.get(i);
				Statistics temp = node.getStatistics();
				correctSum += temp.getCorrectPatterns();
				totalSum += temp.getTotalPatterns();
			}			
		}
		//System.out.println("n interno retornando estatstica. corretos: "+correctSum+", total: "+totalSum);
		Statistics result = new Statistics(correctSum,totalSum);
		return result;
	}
	
	
	
}
