package src.treeController;

import java.util.ArrayList;

import src.hashFunctions.IHashFunction;
import src.hashFunctions.NumStrokes;
import src.mlp.Network;
import src.mlp.NetworkConfigs;
import src.mlp.PatternSymbol;
import src.mlp.statistics.Statistics;

/*Classe que representa a rvore de reconhecimento.*/
public class TreeController {
	
	/*Os parametros da rede neural podem ser modificados alterando-se as trs constantes abaixo.*/
	private int learningPeriods = 100;
	private double learningRate = -0.01;
	private double errorTreshold = 1.0E-1;	
	private ArrayList <IHashFunction> hashFunctions;	
	private ArrayList<PatternSymbol> symbolsForTraining;
	private NetworkConfigs netSize;	
	private InternalNode root;	
	
	/*A estrutura da rvore de reconhecimento pode ser modificada descomentando as hashFunctions abaixo.*/
	public TreeController(NetworkConfigs netSize){
		hashFunctions = new ArrayList <IHashFunction>();
		NumStrokes numberOfStrokesFunction = new NumStrokes();
//		SingleNetwork dummy = new SingleNetwork(); /*Neste caso a rvore ter apenas uma rede neural.*/
//		FirstPoint firsPoint = new FirstPoint();
//		LastPoint lastPoint = new LastPoint();
//		FirstToLastDirection fld = new FirstToLastDirection();
//		Shape shape = new Shape();
//		Density density = new Density();
		
		hashFunctions.add(numberOfStrokesFunction); 
		//hashFunctions.add(shape);		
		//hashFunctions.add(firsPoint);
		//hashFunctions.add(lastPoint); 
		//hashFunctions.add(fld);		
		//hashFunctions.add(density); 
		//hashFunctions.add(dummy);
		this.netSize = netSize;		
		this.netSize.setLearningRate(this.learningRate);
		setRoot(new InternalNode(hashFunctions,0,this.netSize));		
		symbolsForTraining = new ArrayList<PatternSymbol> ();		
	}
	
	
	/*Treina o sistema com os elementos da lista symbolsForTraining*/
	public void train ( ){	
		int recognizedPattern;		
		int actualPattern;
		PatternSymbol ps;
		double error;
		for(int j = 0; j < learningPeriods; j++){
			System.out.println("     Periodo de treinamento: "+j);
			for(int i = 0; i< symbolsForTraining.size();i++){
				//System.out.println("treinando o "+i+"-esimo elemento da lista.");
				ps = symbolsForTraining.get(i);
				actualPattern = ps.getPattern();
				Network net = root.getLeaf(ps).getNet();
				recognizedPattern = net.recognize (ps);
				net.updateWeights(actualPattern , net.getOutputLayerOutput());
				error = net.calculateError(actualPattern );
				while (error > errorTreshold || recognizedPattern != actualPattern ){					
					recognizedPattern = net.recognize (ps);
					net.updateWeights(actualPattern  ,net.getOutputLayerOutput());
					error = net.calculateError(actualPattern);

				}
			}
		}		
	}
	
	/*Reconhece todos os elementos de list. Retorna a taxa de reconhecimento*/
	public double recognizeAll (ArrayList<PatternSymbol> list){
		for(int i=0;i<list.size();i++){			
			PatternSymbol ps = list.get(i) ;			
			Network net = root.getLeaf(ps).getNet();			
			int recognizedSymbol = net.recognize(ps);			
			int actualSymbol =ps.getPattern();			
			net.getConfusionMatrix().addResult(actualSymbol,recognizedSymbol);		
			
		}
		Statistics wholeTreeStatistics = root.getStatistics();
		double recognitionRate = (double)wholeTreeStatistics.getCorrectPatterns()/(double)wholeTreeStatistics.getTotalPatterns();		
		System.out.printf("Taxa de reconhecimento: %5.1f%%\n",recognitionRate*100);
		return recognitionRate*100;		
	}	
	
	public void trainAll(ArrayList<PatternSymbol> list){
		for(int i = 0; i<list.size();i++)
			addPatternSymbol(list.get(i));		
		train();
	}
		
	public void addPatternSymbol(PatternSymbol ps){
		symbolsForTraining.add(ps);		
	}

	public void setRoot(InternalNode root) {
		this.root = root;
	}
}
